﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Security.Authentication;
using System.IO;
using System.Xml.Linq;

using EPiServer;

using Nansen.Viddler.Properties;
using Nansen.Viddler.Hosting;
using System.Web;

namespace Nansen.Viddler
{
    public struct ViddlerMethod
    {
        public const string GetInfo = "viddler.api.getInfo";
        public const string AuthenticateUser = "viddler.users.auth";
        public const string GetUserProfile = "viddler.users.getProfile";
        public const string SetUserProfile = "viddler.users.setProfile";
        public const string SetUserOptions = "viddler.users.setOptions";
        public const string GetRecordToken = "viddler.videos.getRecordToken";
        public const string UploadVideo = "viddler.videos.upload";
        public const string DeleteVideo = "viddler.videos.delete";
        public const string GetVideoStatus = "viddler.videos.getStatus";
        public const string GetVideoDetails = "viddler.videos.getDetails";
        public const string SetVideoDetails = "viddler.videos.setDetails";
        public const string GetVideoDetailsByUrl = "viddler.videos.getDetailsByUrl";
        public const string SetPermalink = "viddler.videos.setPermalink";
        public const string GetVideosByUser = "viddler.videos.getByUser";
        public const string GetVideosByTag = "viddler.videos.getByTag";
        public const string GetFeaturedVideos = "viddler.videos.getFeatured";
        public const string AddComment = "viddler.videos.comments.add";
        public const string RemoveComment = "viddler.videos.comments.remove";
    }

    public class Viddler
    {
        public IEnumerable<ViddlerVideo> GetVideos()
        {
            string endpoint = string.Format(Settings.Default.ApiUrl, ViddlerMethod.GetVideosByUser, Settings.Default.ApiKey);
            string sessionId = this.GetAuthenticatedSessionId();
            
            endpoint = UriSupport.AddQueryString(endpoint, "user", Settings.Default.Username);
            endpoint = UriSupport.AddQueryString(endpoint, "sessionid", sessionId);

            string xml = this.GetMessage(endpoint, "GET");

            XDocument doc = XDocument.Parse(xml);

            return doc.Descendants("video_list").Elements("video").Select(element => new ViddlerVideo(element.ToString()));
        }

        public ViddlerVideo GetDetails(string videoId)
        {
            string endpoint = string.Format(Settings.Default.ApiUrl, ViddlerMethod.GetVideoDetails, Settings.Default.ApiKey);
            string sessionId = this.GetAuthenticatedSessionId();
            
            endpoint = UriSupport.AddQueryString(endpoint, "video_id", videoId);
            endpoint = UriSupport.AddQueryString(endpoint, "sessionid", sessionId);

            string xml = this.GetMessage(endpoint, "GET");

            return new ViddlerVideo(xml);
        }

        public void SetDetails(ViddlerVideo video)
        {
            string endpoint = string.Format(Settings.Default.ApiUrl, ViddlerMethod.SetVideoDetails, Settings.Default.ApiKey);
            string sessionId = this.GetAuthenticatedSessionId();

            endpoint = UriSupport.AddQueryString(endpoint, "video_id", video.Id);
            endpoint = UriSupport.AddQueryString(endpoint, "sessionid", sessionId);
            endpoint = UriSupport.AddQueryString(endpoint, "title", HttpUtility.UrlEncode(video.Title));
            endpoint = UriSupport.AddQueryString(endpoint, "description", HttpUtility.UrlEncode(video.Description));

            string tags = string.Empty;

            foreach (string tag in video.Tags)
            {
                tags += tag.Trim() + ",";
            }

            tags = !string.IsNullOrEmpty(tags) ? tags.Remove(tags.Length - 1) : string.Empty;

            endpoint = UriSupport.AddQueryString(endpoint, "tags", tags);

            this.GetMessage(endpoint, "GET");
        }

        public void Delete(string videoId)
        {
            string endpoint = string.Format(Settings.Default.ApiUrl, ViddlerMethod.DeleteVideo, Settings.Default.ApiKey);
            string sessionId = this.GetAuthenticatedSessionId();

            endpoint = UriSupport.AddQueryString(endpoint, "video_id", videoId);
            endpoint = UriSupport.AddQueryString(endpoint, "sessionid", sessionId);
            this.GetMessage(endpoint, "POST");
        }

        public ViddlerVideo Upload(string title, string tags, string description, Stream fileData)
        {
            string sessionId = this.GetAuthenticatedSessionId();

            string endpoint = string.Format(Settings.Default.ApiUrl, ViddlerMethod.UploadVideo, Settings.Default.ApiKey);

            var request = WebRequest.Create(endpoint) as HttpWebRequest;
            request.ContentType = "multipart/form-data; boundary=" + PostData.Boundary;
            request.Method = "POST";
            request.ContentLength = fileData.Length;

            var postData = new PostData();

            postData.Params.Add(new PostDataParameter("sessionId", sessionId));
            postData.Params.Add(new PostDataParameter("title", title));
            postData.Params.Add(new PostDataParameter("tags", tags));
            postData.Params.Add(new PostDataParameter("description", description));
            postData.Params.Add(new PostDataParameter("make_public", "1"));

            postData.Params.Add(new PostDataFile("file", title, fileData));

            byte[] buffer = postData.GetPostData();

            request.ContentLength = buffer.Length;

            using (Stream requestStream = request.GetRequestStream())
            {
                requestStream.Write(buffer, 0, buffer.Length);
                requestStream.Close();
            }

            var response = (HttpWebResponse)request.GetResponse();

            string xml = new StreamReader(response.GetResponseStream()).ReadToEnd();

            var video = new ViddlerVideo(xml);

            HttpContext.Current.Items["tempViddlerVideoId"] = video.Id;

            return video;
        }

        private string GetAuthenticatedSessionId()
        {
            string endpoint = string.Format(Settings.Default.ApiUrl, ViddlerMethod.AuthenticateUser, Settings.Default.ApiKey);
            endpoint = UriSupport.AddQueryString(endpoint, "user", Settings.Default.Username);
            endpoint = UriSupport.AddQueryString(endpoint, "password", Settings.Default.Password);
            endpoint = UriSupport.AddQueryString(endpoint, "get_record_token", "0");

            string xml = this.GetMessage(endpoint, "GET");
            XDocument doc = XDocument.Parse(xml);

            return doc.Descendants("auth").Elements("sessionid").Single().Value;
        }

        private string GetMessage(string endPoint, string method)
        {
            var request = (HttpWebRequest)WebRequest.Create(endPoint);

            request.Method = method;
            request.ContentLength = 0;
            request.ContentType = "text/xml";

            using (var response = (HttpWebResponse)request.GetResponse())
            {
                var responseValue = string.Empty;

                if (response.StatusCode != HttpStatusCode.OK)
                {
                    string message = string.Format("POST failed. Received HTTP {0}", response.StatusCode);
                    throw new ApplicationException(message);
                }

                using (var responseStream = response.GetResponseStream())
                {
                    using (var reader = new StreamReader(responseStream))
                    {
                        responseValue = reader.ReadToEnd();
                    }
                }

                return responseValue;
            }
        }
    }
}